`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/03/19 16:30:11
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

module topmodule(
    D_ssd,
    ssd_ctl,
    clk,
    //rst_n,
    LED_out,
    pb_in,
    pb_mode
    );

    output [7:0] D_ssd;
    output [3:0] ssd_ctl;
    output [15:0] LED_out;

    input clk;
    //input rst_n;
    input pb_in;
    input pb_mode;
    
    wire clk_1Hz;
    wire clk_100Hz;
    wire [1:0] clk_ssd_ctl;

    wire pb_in_debounced;
    wire pb_mode_debounced;
    wire pb_press_long;

    wire pb_out_1_pulse;
    wire pb_out_mode_pulse;

    wire count_enable;
    wire select_mode;
    wire current_state;

    wire rst_count;

    wire [3:0] a_digit_0, a_digit_1;
    wire [3:0] b_digit_0, b_digit_1, b_digit_2;

    wire a_out_LED, b_out_LED;
    wire LED_in;

    wire [3:0] a_ssd_in, a_ssd_ctl;
    wire [3:0] b_ssd_in, b_ssd_ctl;

    wire [3:0] ssd_in;
   

    clock_generator U0(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk_100Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst(rst_count)   //low-active reset
);
    debounce U1( 
    .rst(rst_count), 
    .clk(clk_100Hz), 
    .pb_in(pb_in),
    .pb_debounced(pb_in_debounced)
);
    debounce U2( 
    .rst(rst_count), 
    .clk(clk_100Hz), 
    .pb_in(pb_mode),
    .pb_debounced(pb_mode_debounced)
);
    press_long U3( 
    .pb_press_long(pb_press_long),
    .rst_count(rst_count),
    .rst(rst_count),
    .pb_debounce_in(pb_in_debounced),
    .clk(clk)
);
    one_pulse U4( 
    .rst(rst_count), 
    .clk(clk_100Hz), 
    .in_trig(pb_press_long),
    .out_pulse(pb_out_1_pulse) 
);
    one_pulse U5( 
    .rst(rst_count), 
    .clk(clk_100Hz), 
    .in_trig(pb_mode_debounced),
    .out_pulse(pb_out_mode_pulse) 
);
    fsm U6(
    .count_enable(count_enable),
    .select_mode(select_mode),
    .in({pb_out_mode_pulse, pb_out_1_pulse}), //串接
    .clk(clk_100Hz),
    .rst(rst_count),
    .curr_state(current_state)
);

    BCD_down_2d U7(
    .digit_1(a_digit_1),
    .digit_0(a_digit_0),
    .clk(clk_1Hz),
    .rst(rst_count),
    .en(count_enable),
    .LED_out_to_ctl(a_out_LED)
    );

    BCD_down_3d U8(
    .digit_2(b_digit_2),
    .digit_1(b_digit_1),
    .digit_0(b_digit_0),
    .clk(clk_1Hz),
    .rst(rst_count),
    .en(count_enable),
    .LED_out_to_ctl(b_out_LED)
    );

 
    scan_freq_ctl U9(
    .ssd_in(a_ssd_in),
    .ssd_ctl(a_ssd_ctl),
    .in_0(a_digit_0),
    .in_1(a_digit_1),
    .in_2(4'b0000),
    .clk_ctl_en(clk_ssd_ctl),
    .rst(rst_count)
    );

    scan_freq_ctl U10(
    .ssd_in(b_ssd_in),
    .ssd_ctl(b_ssd_ctl),
    .in_0(b_digit_0),
    .in_1(b_digit_1),
    .in_2(b_digit_2),
    .clk_ctl_en(clk_ssd_ctl),
    .rst(rst_count)
    );
    
    mode_sel_ctl U11( 
    .select_mode(select_mode),
    .a_ssd_ctl(a_ssd_ctl),
    .b_ssd_ctl(b_ssd_ctl),
    .a_ssd_in(a_ssd_in),
    .b_ssd_in(b_ssd_in),
    .a_out_LED(a_out_LED),
    .b_out_LED(b_out_LED),
    .LED_into_LED_ctl(LED_in),
    .ssd_in(ssd_in),
    .ssd_ctl_out(ssd_ctl)
);
    display U12(.D_ssd(D_ssd), .i(ssd_in));
    
    LED_ctl U13( 
    .current_state(current_state),
    .LED_in(LED_in),
    .LED_out(LED_out)
);


    
endmodule
